package com.usky.aspect;

import com.alibaba.fastjson.JSON;
import com.usky.annotion.SysLog;
import com.usky.entity.TbLogEntity;
import com.usky.entity.TbUserEntity;
import com.usky.service.SysLogService;
import com.usky.utils.IPUtils;
import com.usky.utils.Result;
import org.apache.shiro.SecurityUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.List;

/**
 * @author laowo
 * @version v1.0
 * @date 2020/11/10 14:41
 * @description 切面类
 **/
@Aspect
@Component
public class LogAspect {
    @Autowired
    SysLogService sysLogServiceimpl;
    /**
     * 切面
     */
    @Pointcut("@annotation(com.usky.annotion.SysLog)")
    public void logPoint() {

    }
    @Around("logPoint()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        long beginTime = System.currentTimeMillis();
        //执行方法
        Object result = point.proceed();
        //执行时长(毫秒)
        long time = System.currentTimeMillis() - beginTime;
        /**
         * 保存日志
         */
        //构建日志实体
        TbLogEntity logEntity = new TbLogEntity();
        logEntity.setTime(time + "");
        //请求方法
        MethodSignature signature = (MethodSignature) point.getSignature();
        Method method = signature.getMethod();
        //获日志注解
        SysLog sysLog = method.getAnnotation(SysLog.class);
        if (sysLog != null) {
            //注解的描述
            logEntity.setOperation(sysLog.value());
        }
        //方法名
        String className = point.getTarget().getClass().getName();
        String methodName = method.getName();
        logEntity.setMethod(className + "." + methodName);
        //请求参数
        Object[] args = point.getArgs();
        List<Object> objects = Arrays.asList(args);
        logEntity.setParams(JSON.toJSONString(objects));
        //请求端Ip
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String ipAddr = IPUtils.getIpAddr(request);
        logEntity.setIp(ipAddr);
        //用户名 用户Id
        TbUserEntity user = (TbUserEntity) SecurityUtils.getSubject().getPrincipal();
        if (user==null){
           return new Result(false,"用户未登录！",null);
        }
        logEntity.setUserId(user.getId());
        logEntity.setUserName(user.getUserName());
        //创建日期
        logEntity.setCreatDate(new Timestamp(System.currentTimeMillis()));
        //保存日志
        sysLogServiceimpl.save(logEntity);
        return result;
    }
}