package com.usky.config.shiro;

import com.usky.entity.TbUserEntity;
import com.usky.service.userService.UserService;
import org.apache.shiro.authc.*;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.util.ByteSource;
import org.apache.shiro.util.SimpleByteSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;

import java.util.Set;

/**
 * @author laowo
 * @version v1.0
 * @date 2020/11/3 10:32
 * @description 自定义Realm
 **/

public class MyRealm extends AuthorizingRealm {
    Logger logger = LoggerFactory.getLogger(MyRealm.class);
    @Autowired
    @Lazy
    private UserService userServiceImpl;

    /*
     * 授权逻辑
     * */
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principalCollection) {
        SimpleAuthorizationInfo simpleAuthorizationInfo = new SimpleAuthorizationInfo();
        //获取用户名
        TbUserEntity userEntity = (TbUserEntity) principalCollection.getPrimaryPrincipal();//（认证时构造）
        String username = userEntity.getUserName();
        //从数据库查询用户的权限和角色
        //角色查询
        Set<String> roles = userServiceImpl.queryUseRoleByUserName(username);
        if (roles != null && roles.size() > 0) {
            simpleAuthorizationInfo.addRoles(roles);
        }
        //权限查询
        Set<String> permissions = userServiceImpl.findPermissionsByUsername(username);
        if (permissions != null && permissions.size() > 0) {
            simpleAuthorizationInfo.addStringPermissions(permissions);
        }
        return simpleAuthorizationInfo;
    }

    /*
     * 认证逻辑
     * */
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken authenticationToken) throws AuthenticationException {
        UsernamePasswordToken token = (UsernamePasswordToken) authenticationToken;
        String username = token.getUsername();
        TbUserEntity userEntity = userServiceImpl.queryUserByUserName(username);
        if (userEntity == null) {
            return null;
        }
        ByteSource byteSource = new SimpleByteSource(userEntity.getPrivateSalt());
        AuthenticationInfo authenticationInfo = new SimpleAuthenticationInfo(userEntity, userEntity.getPassword(), byteSource, getName());
        return authenticationInfo;
    }
}
