package com.usky.config.shiro;

import org.apache.shiro.authc.credential.CredentialsMatcher;
import org.apache.shiro.authc.credential.HashedCredentialsMatcher;
import org.apache.shiro.mgt.SecurityManager;
import org.apache.shiro.realm.Realm;
import org.apache.shiro.spring.security.interceptor.AuthorizationAttributeSourceAdvisor;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.springframework.aop.framework.autoproxy.DefaultAdvisorAutoProxyCreator;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Lazy;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @author laowo
 * @version v1.0
 * @date 2020/11/3 9:20
 * @description TODO
 **/


@Configuration
public class ShiroConfig {
    //创建ShiroFilterFactoryBean
    @Bean
    public ShiroFilterFactoryBean getShiroFilterFactoryBean() {
        ShiroFilterFactoryBean shiroFilterFactoryBean = new ShiroFilterFactoryBean();
        //设置安全管理器
        shiroFilterFactoryBean.setSecurityManager(securityManager());
        /*
        anon 无需认证访问
        authc 认证访问
        perms 授权访问
        role 角色授权访问
         */
        Map<String, String> hashMap = new LinkedHashMap<>();
        shiroFilterFactoryBean.setLoginUrl("/");
        hashMap.put("/login/toLogin", "anon");
        hashMap.put("/", "anon");
        //swagger2
        hashMap.put("/swagger-ui.html", "anon");
        hashMap.put("/swagger-resources", "anon");
        hashMap.put("/swagger-resources/configuration/security", "anon");
        hashMap.put("/swagger-resources/configuration/ui", "anon");
        hashMap.put("/v2/api-docs", "anon");
        hashMap.put("/webjars/springfox-swagger-ui/**", "anon");
        //  hashMap.put("/user/queryUserAll", "anon");
        //  hashMap.put("/*", "authc");
        hashMap.put("/login/loginOut", "logout");
        shiroFilterFactoryBean.setFilterChainDefinitionMap(hashMap);
        return shiroFilterFactoryBean;
    }
    //创建DefaultWebSecurityManager
    @Bean("SecurityManager")
    public SecurityManager securityManager() {
        DefaultWebSecurityManager defaultWebSecurityManager = new DefaultWebSecurityManager();
        //     DefaultWebSecurityManager defaultWebSecurityManager = SpringUtil.getBean(DefaultWebSecurityManager.class);
        //关联Reaml
        // ApplicationContext().getBean()
        defaultWebSecurityManager.setRealm(getRealm());
        return defaultWebSecurityManager;
    }
    @Bean(name = "Realm")
    @Lazy
    public Realm getRealm() {
        MyRealm myRealm = new MyRealm();
        //设置密码匹配器
        myRealm.setCredentialsMatcher(credentialsMatcher());
        return myRealm;
    }
    //创建密码匹配器
    @Bean
    public CredentialsMatcher credentialsMatcher() {
        HashedCredentialsMatcher hashedCredentialsMatcher = new HashedCredentialsMatcher();
        hashedCredentialsMatcher.setHashAlgorithmName("md5");
        hashedCredentialsMatcher.setHashIterations(1);
        return hashedCredentialsMatcher;
    }
    /**
     * 注解支持：
     */
    @Bean
    @ConditionalOnMissingBean
    public DefaultAdvisorAutoProxyCreator defaultAdvisorAutoProxyCreator() {
        DefaultAdvisorAutoProxyCreator defaultAAP = new DefaultAdvisorAutoProxyCreator();
        defaultAAP.setProxyTargetClass(true);
        return defaultAAP;
    }
    @Bean
    public AuthorizationAttributeSourceAdvisor authorizationAttributeSourceAdvisor(SecurityManager securityManager) {
        AuthorizationAttributeSourceAdvisor authorizationAttributeSourceAdvisor = new AuthorizationAttributeSourceAdvisor();
        authorizationAttributeSourceAdvisor.setSecurityManager(securityManager);
        return authorizationAttributeSourceAdvisor;
    }
}
