package com.usky.utils;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.util.List;

/**
 * @author laowo
 * @version v1.0
 * @date 2020/2/24 9:01
 * @description TODO
 **/
@ApiModel(value="分页返回结果集")
public class Page<T> {
    @ApiModelProperty("每页显示条数")
    private int pageSize; //每页显示条数
    @ApiModelProperty("总条数")
    private int totalCount; //总条数
    @ApiModelProperty("开始条数")
    private int start; //开始条数
    @ApiModelProperty("当前页")
    private int pageNo;//当前页
    @ApiModelProperty("总页数")
    private int totalPages; //总页数
    @ApiModelProperty("数据")
    private List<T> pageList;//数据

    public Page(int totalCount, Integer pageSize) {
        if (pageSize == null) {
            this.pageSize = 20;
        } else {
            this.pageSize = pageSize;
        }
        this.totalCount = totalCount;
    }

    /**
     * 获取下一条
     *
     * @return
     */
    public int getCurrentPageNo() {
        return start / pageSize + 1;
    }

    /**
     * 是否有下一条
     *
     * @return
     */
    public boolean getHasNextPage() {
        return getCurrentPageNo() < totalPages;
    }

    /**
     * 当前页是否大于一
     *
     * @return
     */
    public boolean getHasPavPage() {
        return getCurrentPageNo() > 1;
    }

    /**
     * 获取总页数
     *
     * @return
     */
    public int getTotalPages() {
        totalPages = totalCount / pageSize;

        if (totalCount % pageSize != 0) {
            totalPages++;
        }

        return totalPages;
    }

    public int getStart(int pageNo) {
        this.pageNo = pageNo;
        if (pageNo < 1) {
            pageNo = 1;
        }
        start = (pageNo - 1) * pageSize;
        return start;
    }

    public int getPageSize() {
        return pageSize;
    }

    public void setPageSize(int pageSize) {
        this.pageSize = pageSize;
    }

    public int getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(int totalCount) {
        this.totalCount = totalCount;
    }

    public void setStart(int start) {
        this.start = start;
    }

    public List<T> getPageList() {
        return pageList;
    }

    public void setPageList(List<T> pageList) {
        this.pageList = pageList;
    }

    public void setTotalPages(int totalPages) {
        this.totalPages = totalPages;
    }

    public int getPageNo() {
        return pageNo;
    }

    public void setPageNo(int pageNo) {
        this.pageNo = pageNo;
    }
}
