package com.usky.utils;

import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;

/**
 * @author laowo
 * @version v1.0
 * @date 2020/2/20 14:59
 * @description 接口统一返回结果
 **/
@ApiModel(value="统一操作结果集")
@Data
public class ResultBase<T> implements Serializable {
    private static final long serialVersionUID = 1L;
    @ApiModelProperty(value = "版本号")
    private String VER = "1.00";
    /**
     * 成功标志
     */
    @ApiModelProperty(value = "成功标志")
    private boolean success = true;

    /**
     * 返回处理消息
     */
    @ApiModelProperty(value = "返回处理消息")
    private String message = "操作成功！";

    /**
     * 返回代码
     */
    @ApiModelProperty(value = "返回代码")
    private Integer code = 0;

    /**
     * 返回数据对象 data
     */
    @ApiModelProperty(value = "返回数据对象")
    private T data;

    /**
     * 时间戳
     */
    @ApiModelProperty(value = "时间戳")
    private long timestamp = System.currentTimeMillis();
    public ResultBase() {

    }

    public ResultBase<T> success(String message) {
        this.message = message;
        this.code = 200;
        this.success = true;
        return this;
    }

    public static ResultBase<Object> ok() {
        ResultBase<Object> r = new ResultBase<Object>();
        r.setSuccess(true);
        r.setCode(200);
        r.setMessage("成功");
        return r;
    }
    public static ResultBase<Object> ok(String msg) {
        ResultBase<Object> r = new ResultBase<Object>();
        r.setSuccess(true);
        r.setCode(200);
        r.setMessage(msg);
        return r;
    }


    public static ResultBase<Object> ok(Object data) {
        ResultBase<Object> r = new ResultBase<Object>();
        r.setSuccess(true);
        r.setCode(200);
        r.setData(data);
        return r;
    }

    public static<T> ResultBase<T> OK() {
        ResultBase<T> r = new ResultBase<T>();
        r.setSuccess(true);
        r.setCode(200);
        r.setMessage("成功");
        return r;
    }

    public static<T> ResultBase<T> OK(T data) {
        ResultBase<T> r = new ResultBase<T>();
        r.setSuccess(true);
        r.setCode(200);
        r.setData(data);
        return r;
    }

    public static<T> ResultBase<T> OK(String msg, T data) {
        ResultBase<T> r = new ResultBase<T>();
        r.setSuccess(true);
        r.setCode(200);
        r.setMessage(msg);
        r.setData(data);
        return r;
    }

    public static ResultBase<Object> error(String msg) {
        return error(500, msg);
    }

    public static ResultBase<Object> error(int code, String msg) {
        ResultBase<Object> r = new ResultBase<Object>();
        r.setCode(code);
        r.setMessage(msg);
        r.setSuccess(false);
        return r;
    }

    public ResultBase<T> error500(String message) {
        this.message = message;
        this.code = 500;
        this.success = false;
        return this;
    }
    /**
     * 无权限访问返回结果
     */
    public static ResultBase<Object> noauth(String msg) {
        return error(510, msg);
    }


}